@@ -6,6 +6,7 @@ module Data.Tuple.HT (
    swap,
    sortPair,
    forcePair,
+   double,
 
    -- * Triple
    fst3,
@@ -17,11 +18,20 @@ module Data.Tuple.HT (
    mapThd3,
    curry3,
    uncurry3,
+   triple,
    ) where
 
 import Data.Tuple.Lazy
 
 
+{-# INLINE double #-}
+double :: a -> (a,a)
+double a = (a,a)
+
+{-# INLINE triple #-}
+triple :: a -> (a,a,a)
+triple a = (a,a,a)
+
 {-# INLINE fst3 #-}
 fst3 :: (a,b,c) -> a
 fst3 (x,_,_) = x
